@@ -84,6 +84,7 @@ INSTANTIATE_TEST_CASE_P(
         "cppBad2.vert",
         "cppComplexExpr.vert",
         "cppDeepNest.frag",
+        "cppPassMacroName.frag",
         "badChars.frag",
         "pointCoord.frag",
         "array.frag",
@@ -160,7 +161,7 @@ INSTANTIATE_TEST_CASE_P(
         "460.vert",
         "dce.frag",
         "atomic_uint.frag",
-//        "implicitInnerAtomicUint.frag",
+        "implicitInnerAtomicUint.frag",
         "aggOps.frag",
         "always-discard.frag",
         "always-discard2.frag",
