@@ -53,15 +53,15 @@ implicit_type_map = {
 }
 # Function/struct value mappings, shifted up FUNC_STRUCT_SHIFT bits in final ID
 func_struct_id_map = {
-'VkAcquireNextImageInfoKHX' : 0,
+'VkAcquireNextImageInfo' : 0,
 'VkAllocationCallbacks' : 1,
-'VkAndroidSurfaceCreateInfoKHR' : 2,
+'VkAndroidSurfaceCreateInfo' : 2,
 'VkApplicationInfo' : 3,
 'VkAttachmentDescription' : 4,
 'VkAttachmentReference' : 5,
-'VkBindBufferMemoryInfoKHR' : 6,
-'VkBindImageMemoryInfoKHR' : 7,
-'VkBindImageMemorySwapchainInfoKHX' : 8,
+'VkBindBufferMemoryInfo' : 6,
+'VkBindImageMemoryInfo' : 7,
+'VkBindImageMemorySwapchainInfo' : 8,
 'VkBindSparseInfo' : 9,
 'VkBufferCreateInfo' : 10,
 'VkBufferImageCopy' : 11,
@@ -79,7 +79,7 @@ func_struct_id_map = {
 'VkComponentMapping' : 23,
 'VkComputePipelineCreateInfo' : 24,
 'VkCopyDescriptorSet' : 25,
-'VkD3D12FenceSubmitInfoKHR' : 26,
+'VkD3D12FenceSubmitInfo' : 26,
 'VkDebugMarkerMarkerInfoEXT' : 27,
 'VkDebugMarkerObjectNameInfoEXT' : 28,
 'VkDebugMarkerObjectTagInfoEXT' : 29,
@@ -94,37 +94,37 @@ func_struct_id_map = {
 'VkDescriptorSetAllocateInfo' : 38,
 'VkDescriptorSetLayoutBinding' : 39,
 'VkDescriptorSetLayoutCreateInfo' : 40,
-'VkDescriptorUpdateTemplateCreateInfoKHR' : 41,
-'VkDescriptorUpdateTemplateEntryKHR' : 42,
+'VkDescriptorUpdateTemplateCreateInfo' : 41,
+'VkDescriptorUpdateTemplateEntry' : 42,
 'VkDeviceCreateInfo' : 43,
 'VkDeviceEventInfoEXT' : 44,
 'VkDeviceGeneratedCommandsFeaturesNVX' : 45,
 'VkDeviceGeneratedCommandsLimitsNVX' : 46,
-'VkDeviceGroupBindSparseInfoKHX' : 47,
-'VkDeviceGroupCommandBufferBeginInfoKHX' : 48,
-'VkDeviceGroupDeviceCreateInfoKHX' : 49,
-'VkDeviceGroupPresentInfoKHX' : 50,
-'VkDeviceGroupRenderPassBeginInfoKHX' : 51,
-'VkDeviceGroupSubmitInfoKHX' : 52,
-'VkDeviceGroupSwapchainCreateInfoKHX' : 53,
+'VkDeviceGroupBindSparseInfo' : 47,
+'VkDeviceGroupCommandBufferBeginInfo' : 48,
+'VkDeviceGroupDeviceCreateInfo' : 49,
+'VkDeviceGroupPresentInfo' : 50,
+'VkDeviceGroupRenderPassBeginInfo' : 51,
+'VkDeviceGroupSubmitInfo' : 52,
+'VkDeviceGroupSwapchainCreateInfo' : 53,
 'VkDeviceQueueCreateInfo' : 54,
 'VkDispatchIndirectCommand' : 55,
 'VkDisplayEventInfoEXT' : 56,
-'VkDisplayModeCreateInfoKHR' : 57,
+'VkDisplayModeCreateInfo' : 57,
 'VkDisplayPowerInfoEXT' : 58,
-'VkDisplayPresentInfoKHR' : 59,
-'VkDisplaySurfaceCreateInfoKHR' : 60,
+'VkDisplayPresentInfo' : 59,
+'VkDisplaySurfaceCreateInfo' : 60,
 'VkDrawIndexedIndirectCommand' : 61,
 'VkDrawIndirectCommand' : 62,
 'VkEventCreateInfo' : 63,
-'VkExportMemoryAllocateInfoKHR' : 64,
+'VkExportMemoryAllocateInfo' : 64,
 'VkExportMemoryAllocateInfoNV' : 65,
-'VkExportMemoryWin32HandleInfoKHR' : 66,
+'VkExportMemoryWin32HandleInfo' : 66,
 'VkExportMemoryWin32HandleInfoNV' : 67,
-'VkExportSemaphoreCreateInfoKHR' : 68,
-'VkExportSemaphoreWin32HandleInfoKHR' : 69,
-'VkExternalMemoryBufferCreateInfoKHR' : 70,
-'VkExternalMemoryImageCreateInfoKHR' : 71,
+'VkExportSemaphoreCreateInfo' : 68,
+'VkExportSemaphoreWin32HandleInfo' : 69,
+'VkExternalMemoryBufferCreateInfo' : 70,
+'VkExternalMemoryImageCreateInfo' : 71,
 'VkExternalMemoryImageCreateInfoNV' : 72,
 'VkFenceCreateInfo' : 73,
 'VkFramebufferCreateInfo' : 74,
@@ -138,23 +138,23 @@ func_struct_id_map = {
 'VkImageSubresource' : 82,
 'VkImageSubresourceLayers' : 83,
 'VkImageSubresourceRange' : 84,
-'VkImageSwapchainCreateInfoKHX' : 85,
+'VkImageSwapchainCreateInfo' : 85,
 'VkImageViewCreateInfo' : 86,
-'VkImportMemoryFdInfoKHR' : 87,
-'VkImportMemoryWin32HandleInfoKHR' : 88,
+'VkImportMemoryFdInfo' : 87,
+'VkImportMemoryWin32HandleInfo' : 88,
 'VkImportMemoryWin32HandleInfoNV' : 89,
-'VkImportSemaphoreFdInfoKHR' : 90,
-'VkImportSemaphoreWin32HandleInfoKHR' : 91,
+'VkImportSemaphoreFdInfo' : 90,
+'VkImportSemaphoreWin32HandleInfo' : 91,
 'VkIndirectCommandsLayoutCreateInfoNVX' : 92,
 'VkIndirectCommandsLayoutTokenNVX' : 93,
 'VkIndirectCommandsTokenNVX' : 94,
 'VkInstanceCreateInfo' : 95,
 'VkMacOSSurfaceCreateInfoMVK' : 96,
 'VkMappedMemoryRange' : 97,
-'VkMemoryAllocateFlagsInfoKHX' : 98,
+'VkMemoryAllocateFlagsInfo' : 98,
 'VkMemoryAllocateInfo' : 99,
 'VkMemoryBarrier' : 100,
-'VkMirSurfaceCreateInfoKHR' : 101,
+'VkMirSurfaceCreateInfo' : 101,
 'VkObjectTableCreateInfoNVX' : 102,
 'VkObjectTableDescriptorSetEntryNVX' : 103,
 'VkObjectTableEntryNVX' : 104,
@@ -163,16 +163,16 @@ func_struct_id_map = {
 'VkObjectTablePushConstantEntryNVX' : 107,
 'VkObjectTableVertexBufferEntryNVX' : 108,
 'VkPhysicalDeviceDiscardRectanglePropertiesEXT' : 109,
-'VkPhysicalDeviceExternalBufferInfoKHR' : 110,
-'VkPhysicalDeviceExternalImageFormatInfoKHR' : 111,
-'VkPhysicalDeviceExternalSemaphoreInfoKHR' : 112,
+'VkPhysicalDeviceExternalBufferInfo' : 110,
+'VkPhysicalDeviceExternalImageFormatInfo' : 111,
+'VkPhysicalDeviceExternalSemaphoreInfo' : 112,
 'VkPhysicalDeviceFeatures' : 113,
-'VkPhysicalDeviceFeatures2KHR' : 114,
-'VkPhysicalDeviceImageFormatInfo2KHR' : 115,
-'VkPhysicalDeviceMultiviewFeaturesKHX' : 116,
-'VkPhysicalDevicePushDescriptorPropertiesKHR' : 117,
-'VkPhysicalDeviceSparseImageFormatInfo2KHR' : 118,
-'VkPhysicalDeviceSurfaceInfo2KHR' : 119,
+'VkPhysicalDeviceFeatures2' : 114,
+'VkPhysicalDeviceImageFormatInfo2' : 115,
+'VkPhysicalDeviceMultiviewFeatures' : 116,
+'VkPhysicalDevicePushDescriptorProperties' : 117,
+'VkPhysicalDeviceSparseImageFormatInfo2' : 118,
+'VkPhysicalDeviceSurfaceInfo2' : 119,
 'VkPipelineCacheCreateInfo' : 120,
 'VkPipelineColorBlendAttachmentState' : 121,
 'VkPipelineColorBlendStateCreateInfo' : 122,
@@ -190,16 +190,16 @@ func_struct_id_map = {
 'VkPipelineViewportStateCreateInfo' : 134,
 'VkPipelineViewportSwizzleStateCreateInfoNV' : 135,
 'VkPipelineViewportWScalingStateCreateInfoNV' : 136,
-'VkPresentInfoKHR' : 137,
-'VkPresentRegionKHR' : 138,
-'VkPresentRegionsKHR' : 139,
+'VkPresentInfo' : 137,
+'VkPresentRegion' : 138,
+'VkPresentRegions' : 139,
 'VkPresentTimesInfoGOOGLE' : 140,
 'VkPushConstantRange' : 141,
 'VkQueryPoolCreateInfo' : 142,
-'VkRectLayerKHR' : 143,
+'VkRectLayer' : 143,
 'VkRenderPassBeginInfo' : 144,
 'VkRenderPassCreateInfo' : 145,
-'VkRenderPassMultiviewCreateInfoKHX' : 146,
+'VkRenderPassMultiviewCreateInfo' : 146,
 'VkSamplerCreateInfo' : 147,
 'VkSemaphoreCreateInfo' : 148,
 'VkShaderModuleCreateInfo' : 149,
@@ -216,31 +216,31 @@ func_struct_id_map = {
 'VkSubpassDescription' : 160,
 'VkSurfaceCapabilities2EXT' : 161,
 'VkSwapchainCounterCreateInfoEXT' : 162,
-'VkSwapchainCreateInfoKHR' : 163,
+'VkSwapchainCreateInfo' : 163,
 'VkValidationFlagsEXT' : 164,
 'VkVertexInputAttributeDescription' : 165,
 'VkVertexInputBindingDescription' : 166,
 'VkViSurfaceCreateInfoNN' : 167,
 'VkViewport' : 168,
 'VkViewportSwizzleNV' : 169,
-'VkWaylandSurfaceCreateInfoKHR' : 170,
-'VkWin32KeyedMutexAcquireReleaseInfoKHR' : 171,
+'VkWaylandSurfaceCreateInfo' : 170,
+'VkWin32KeyedMutexAcquireReleaseInfo' : 171,
 'VkWin32KeyedMutexAcquireReleaseInfoNV' : 172,
-'VkWin32SurfaceCreateInfoKHR' : 173,
+'VkWin32SurfaceCreateInfo' : 173,
 'VkWriteDescriptorSet' : 174,
-'VkXcbSurfaceCreateInfoKHR' : 175,
-'VkXlibSurfaceCreateInfoKHR' : 176,
-'vkAcquireNextImage2KHX' : 177,
-'vkAcquireNextImageKHR' : 178,
+'VkXcbSurfaceCreateInfo' : 175,
+'VkXlibSurfaceCreateInfo' : 176,
+'vkAcquireNextImage2' : 177,
+'vkAcquireNextImage' : 178,
 'vkAcquireXlibDisplayEXT' : 179,
 'vkAllocateCommandBuffers' : 180,
 'vkAllocateDescriptorSets' : 181,
 'vkAllocateMemory' : 182,
 'vkBeginCommandBuffer' : 183,
 'vkBindBufferMemory' : 184,
-'vkBindBufferMemory2KHR' : 185,
+'vkBindBufferMemory2' : 185,
 'vkBindImageMemory' : 186,
-'vkBindImageMemory2KHR' : 187,
+'vkBindImageMemory2' : 187,
 'vkCmdBeginQuery' : 188,
 'vkCmdBeginRenderPass' : 189,
 'vkCmdBindDescriptorSets' : 190,
@@ -260,7 +260,7 @@ func_struct_id_map = {
 'vkCmdDebugMarkerEndEXT' : 204,
 'vkCmdDebugMarkerInsertEXT' : 205,
 'vkCmdDispatch' : 206,
-'vkCmdDispatchBaseKHX' : 207,
+'vkCmdDispatchBase' : 207,
 'vkCmdDispatchIndirect' : 208,
 'vkCmdDraw' : 209,
 'vkCmdDrawIndexed' : 210,
@@ -276,8 +276,8 @@ func_struct_id_map = {
 'vkCmdPipelineBarrier' : 220,
 'vkCmdProcessCommandsNVX' : 221,
 'vkCmdPushConstants' : 222,
-'vkCmdPushDescriptorSetKHR' : 223,
-'vkCmdPushDescriptorSetWithTemplateKHR' : 224,
+'vkCmdPushDescriptorSet' : 223,
+'vkCmdPushDescriptorSetWithTemplate' : 224,
 'vkCmdReserveSpaceForCommandsNVX' : 225,
 'vkCmdResetEvent' : 226,
 'vkCmdResetQueryPool' : 227,
@@ -285,7 +285,7 @@ func_struct_id_map = {
 'vkCmdSetBlendConstants' : 229,
 'vkCmdSetDepthBias' : 230,
 'vkCmdSetDepthBounds' : 231,
-'vkCmdSetDeviceMaskKHX' : 232,
+'vkCmdSetDeviceMask' : 232,
 'vkCmdSetDiscardRectangleEXT' : 233,
 'vkCmdSetEvent' : 234,
 'vkCmdSetLineWidth' : 235,
@@ -298,7 +298,7 @@ func_struct_id_map = {
 'vkCmdUpdateBuffer' : 242,
 'vkCmdWaitEvents' : 243,
 'vkCmdWriteTimestamp' : 244,
-'vkCreateAndroidSurfaceKHR' : 245,
+'vkCreateAndroidSurface' : 245,
 'vkCreateBuffer' : 246,
 'vkCreateBufferView' : 247,
 'vkCreateCommandPool' : 248,
@@ -306,10 +306,10 @@ func_struct_id_map = {
 'vkCreateDebugReportCallbackEXT' : 250,
 'vkCreateDescriptorPool' : 251,
 'vkCreateDescriptorSetLayout' : 252,
-'vkCreateDescriptorUpdateTemplateKHR' : 253,
+'vkCreateDescriptorUpdateTemplate' : 253,
 'vkCreateDevice' : 254,
-'vkCreateDisplayModeKHR' : 255,
-'vkCreateDisplayPlaneSurfaceKHR' : 256,
+'vkCreateDisplayMode' : 255,
+'vkCreateDisplayPlaneSurface' : 256,
 'vkCreateEvent' : 257,
 'vkCreateFence' : 258,
 'vkCreateFramebuffer' : 259,
@@ -320,7 +320,7 @@ func_struct_id_map = {
 'vkCreateIndirectCommandsLayoutNVX' : 264,
 'vkCreateInstance' : 265,
 'vkCreateMacOSSurfaceMVK' : 266,
-'vkCreateMirSurfaceKHR' : 267,
+'vkCreateMirSurface' : 267,
 'vkCreateObjectTableNVX' : 268,
 'vkCreatePipelineCache' : 269,
 'vkCreatePipelineLayout' : 270,
@@ -329,13 +329,13 @@ func_struct_id_map = {
 'vkCreateSampler' : 273,
 'vkCreateSemaphore' : 274,
 'vkCreateShaderModule' : 275,
-'vkCreateSharedSwapchainsKHR' : 276,
-'vkCreateSwapchainKHR' : 277,
+'vkCreateSharedSwapchains' : 276,
+'vkCreateSwapchain' : 277,
 'vkCreateViSurfaceNN' : 278,
-'vkCreateWaylandSurfaceKHR' : 279,
-'vkCreateWin32SurfaceKHR' : 280,
-'vkCreateXcbSurfaceKHR' : 281,
-'vkCreateXlibSurfaceKHR' : 282,
+'vkCreateWaylandSurface' : 279,
+'vkCreateWin32Surface' : 280,
+'vkCreateXcbSurface' : 281,
+'vkCreateXlibSurface' : 282,
 'vkDebugMarkerSetObjectNameEXT' : 283,
 'vkDebugMarkerSetObjectTagEXT' : 284,
 'vkDebugReportMessageEXT' : 285,
@@ -345,7 +345,7 @@ func_struct_id_map = {
 'vkDestroyDebugReportCallbackEXT' : 289,
 'vkDestroyDescriptorPool' : 290,
 'vkDestroyDescriptorSetLayout' : 291,
-'vkDestroyDescriptorUpdateTemplateKHR' : 292,
+'vkDestroyDescriptorUpdateTemplate' : 292,
 'vkDestroyDevice' : 293,
 'vkDestroyEvent' : 294,
 'vkDestroyFence' : 295,
@@ -363,8 +363,8 @@ func_struct_id_map = {
 'vkDestroySampler' : 307,
 'vkDestroySemaphore' : 308,
 'vkDestroyShaderModule' : 309,
-'vkDestroySurfaceKHR' : 310,
-'vkDestroySwapchainKHR' : 311,
+'vkDestroySurface' : 310,
+'vkDestroySwapchain' : 311,
 'vkDeviceWaitIdle' : 312,
 'vkDisplayPowerControlEXT' : 313,
 'vkEndCommandBuffer' : 314,
@@ -372,84 +372,84 @@ func_struct_id_map = {
 'vkEnumerateDeviceLayerProperties' : 316,
 'vkEnumerateInstanceExtensionProperties' : 317,
 'vkEnumerateInstanceLayerProperties' : 318,
-'vkEnumeratePhysicalDeviceGroupsKHX' : 319,
+'vkEnumeratePhysicalDeviceGroups' : 319,
 'vkEnumeratePhysicalDevices' : 320,
 'vkFlushMappedMemoryRanges' : 321,
 'vkFreeCommandBuffers' : 322,
 'vkFreeDescriptorSets' : 323,
 'vkFreeMemory' : 324,
 'vkGetBufferMemoryRequirements' : 325,
-'vkGetDeviceGroupPeerMemoryFeaturesKHX' : 326,
-'vkGetDeviceGroupPresentCapabilitiesKHX' : 327,
-'vkGetDeviceGroupSurfacePresentModesKHX' : 328,
+'vkGetDeviceGroupPeerMemoryFeatures' : 326,
+'vkGetDeviceGroupPresentCapabilities' : 327,
+'vkGetDeviceGroupSurfacePresentModes' : 328,
 'vkGetDeviceMemoryCommitment' : 329,
 'vkGetDeviceProcAddr' : 330,
 'vkGetDeviceQueue' : 331,
-'vkGetDisplayModePropertiesKHR' : 332,
-'vkGetDisplayPlaneCapabilitiesKHR' : 333,
-'vkGetDisplayPlaneSupportedDisplaysKHR' : 334,
+'vkGetDisplayModeProperties' : 332,
+'vkGetDisplayPlaneCapabilities' : 333,
+'vkGetDisplayPlaneSupportedDisplays' : 334,
 'vkGetEventStatus' : 335,
 'vkGetFenceStatus' : 336,
 'vkGetImageMemoryRequirements' : 337,
 'vkGetImageSparseMemoryRequirements' : 338,
 'vkGetImageSubresourceLayout' : 339,
 'vkGetInstanceProcAddr' : 340,
-'vkGetMemoryFdKHR' : 341,
-'vkGetMemoryFdPropertiesKHR' : 342,
-'vkGetMemoryWin32HandleKHR' : 343,
+'vkGetMemoryFd' : 341,
+'vkGetMemoryFdProperties' : 342,
+'vkGetMemoryWin32Handle' : 343,
 'vkGetMemoryWin32HandleNV' : 344,
-'vkGetMemoryWin32HandlePropertiesKHR' : 345,
+'vkGetMemoryWin32HandleProperties' : 345,
 'vkGetPastPresentationTimingGOOGLE' : 346,
-'vkGetPhysicalDeviceDisplayPlanePropertiesKHR' : 347,
-'vkGetPhysicalDeviceDisplayPropertiesKHR' : 348,
-'vkGetPhysicalDeviceExternalBufferPropertiesKHR' : 349,
+'vkGetPhysicalDeviceDisplayPlaneProperties' : 347,
+'vkGetPhysicalDeviceDisplayProperties' : 348,
+'vkGetPhysicalDeviceExternalBufferProperties' : 349,
 'vkGetPhysicalDeviceExternalImageFormatPropertiesNV' : 350,
-'vkGetPhysicalDeviceExternalSemaphorePropertiesKHR' : 351,
+'vkGetPhysicalDeviceExternalSemaphoreProperties' : 351,
 'vkGetPhysicalDeviceFeatures' : 352,
-'vkGetPhysicalDeviceFeatures2KHR' : 353,
+'vkGetPhysicalDeviceFeatures2' : 353,
 'vkGetPhysicalDeviceFormatProperties' : 354,
-'vkGetPhysicalDeviceFormatProperties2KHR' : 355,
+'vkGetPhysicalDeviceFormatProperties2' : 355,
 'vkGetPhysicalDeviceGeneratedCommandsPropertiesNVX' : 356,
 'vkGetPhysicalDeviceImageFormatProperties' : 357,
-'vkGetPhysicalDeviceImageFormatProperties2KHR' : 358,
+'vkGetPhysicalDeviceImageFormatProperties2' : 358,
 'vkGetPhysicalDeviceMemoryProperties' : 359,
-'vkGetPhysicalDeviceMemoryProperties2KHR' : 360,
-'vkGetPhysicalDeviceMirPresentationSupportKHR' : 361,
-'vkGetPhysicalDevicePresentRectanglesKHX' : 362,
+'vkGetPhysicalDeviceMemoryProperties2' : 360,
+'vkGetPhysicalDeviceMirPresentationSupport' : 361,
+'vkGetPhysicalDevicePresentRectangles' : 362,
 'vkGetPhysicalDeviceProperties' : 363,
-'vkGetPhysicalDeviceProperties2KHR' : 364,
+'vkGetPhysicalDeviceProperties2' : 364,
 'vkGetPhysicalDeviceQueueFamilyProperties' : 365,
-'vkGetPhysicalDeviceQueueFamilyProperties2KHR' : 366,
+'vkGetPhysicalDeviceQueueFamilyProperties2' : 366,
 'vkGetPhysicalDeviceSparseImageFormatProperties' : 367,
-'vkGetPhysicalDeviceSparseImageFormatProperties2KHR' : 368,
+'vkGetPhysicalDeviceSparseImageFormatProperties2' : 368,
 'vkGetPhysicalDeviceSurfaceCapabilities2EXT' : 369,
-'vkGetPhysicalDeviceSurfaceCapabilities2KHR' : 370,
-'vkGetPhysicalDeviceSurfaceCapabilitiesKHR' : 371,
-'vkGetPhysicalDeviceSurfaceFormats2KHR' : 372,
-'vkGetPhysicalDeviceSurfaceFormatsKHR' : 373,
-'vkGetPhysicalDeviceSurfacePresentModesKHR' : 374,
-'vkGetPhysicalDeviceSurfaceSupportKHR' : 375,
-'vkGetPhysicalDeviceWaylandPresentationSupportKHR' : 376,
-'vkGetPhysicalDeviceWin32PresentationSupportKHR' : 377,
-'vkGetPhysicalDeviceXcbPresentationSupportKHR' : 378,
-'vkGetPhysicalDeviceXlibPresentationSupportKHR' : 379,
+'vkGetPhysicalDeviceSurfaceCapabilities2' : 370,
+'vkGetPhysicalDeviceSurfaceCapabilities' : 371,
+'vkGetPhysicalDeviceSurfaceFormats2' : 372,
+'vkGetPhysicalDeviceSurfaceFormats' : 373,
+'vkGetPhysicalDeviceSurfacePresentModes' : 374,
+'vkGetPhysicalDeviceSurfaceSupport' : 375,
+'vkGetPhysicalDeviceWaylandPresentationSupport' : 376,
+'vkGetPhysicalDeviceWin32PresentationSupport' : 377,
+'vkGetPhysicalDeviceXcbPresentationSupport' : 378,
+'vkGetPhysicalDeviceXlibPresentationSupport' : 379,
 'vkGetPipelineCacheData' : 380,
 'vkGetQueryPoolResults' : 381,
 'vkGetRandROutputDisplayEXT' : 382,
 'vkGetRefreshCycleDurationGOOGLE' : 383,
 'vkGetRenderAreaGranularity' : 384,
-'vkGetSemaphoreFdKHR' : 385,
-'vkGetSemaphoreWin32HandleKHR' : 386,
+'vkGetSemaphoreFd' : 385,
+'vkGetSemaphoreWin32Handle' : 386,
 'vkGetSwapchainCounterEXT' : 387,
-'vkGetSwapchainImagesKHR' : 388,
-'vkGetSwapchainStatusKHR' : 389,
-'vkImportSemaphoreFdKHR' : 390,
-'vkImportSemaphoreWin32HandleKHR' : 391,
+'vkGetSwapchainImages' : 388,
+'vkGetSwapchainStatus' : 389,
+'vkImportSemaphoreFd' : 390,
+'vkImportSemaphoreWin32Handle' : 391,
 'vkInvalidateMappedMemoryRanges' : 392,
 'vkMapMemory' : 393,
 'vkMergePipelineCaches' : 394,
 'vkQueueBindSparse' : 395,
-'vkQueuePresentKHR' : 396,
+'vkQueuePresent' : 396,
 'vkQueueSubmit' : 397,
 'vkQueueWaitIdle' : 398,
 'vkRegisterDeviceEventEXT' : 399,
@@ -463,33 +463,33 @@ func_struct_id_map = {
 'vkResetFences' : 407,
 'vkSetEvent' : 408,
 'vkSetHdrMetadataEXT' : 409,
-'vkTrimCommandPoolKHR' : 410,
+'vkTrimCommandPool' : 410,
 'vkUnmapMemory' : 411,
 'vkUnregisterObjectsNVX' : 412,
-'vkUpdateDescriptorSetWithTemplateKHR' : 413,
+'vkUpdateDescriptorSetWithTemplate' : 413,
 'vkUpdateDescriptorSets' : 414,
 'vkWaitForFences' : 415,
-'VkPhysicalDeviceProperties2KHR' : 416,
-'VkFormatProperties2KHR' : 417,
-'VkImageFormatProperties2KHR' : 418,
-'VkPhysicalDeviceMemoryProperties2KHR' : 419,
-'VkSurfaceCapabilities2KHR' : 420,
-'VkDeviceGroupPresentCapabilitiesKHX' : 421,
-'VkExternalBufferPropertiesKHR' : 422,
-'VkMemoryWin32HandlePropertiesKHR' : 423,
-'VkMemoryFdPropertiesKHR' : 424,
-'VkExternalSemaphorePropertiesKHR' : 425,
-'VkQueueFamilyProperties2KHR' : 426,
-'VkSparseImageFormatProperties2KHR' : 427,
-'VkSurfaceFormat2KHR' : 428,
+'VkPhysicalDeviceProperties2' : 416,
+'VkFormatProperties2' : 417,
+'VkImageFormatProperties2' : 418,
+'VkPhysicalDeviceMemoryProperties2' : 419,
+'VkSurfaceCapabilities2' : 420,
+'VkDeviceGroupPresentCapabilities' : 421,
+'VkExternalBufferProperties' : 422,
+'VkMemoryWin32HandleProperties' : 423,
+'VkMemoryFdProperties' : 424,
+'VkExternalSemaphoreProperties' : 425,
+'VkQueueFamilyProperties2' : 426,
+'VkSparseImageFormatProperties2' : 427,
+'VkSurfaceFormat2' : 428,
 'VkTextureLODGatherFormatPropertiesAMD' : 429,
-'VkPhysicalDeviceMultiviewPropertiesKHX' : 430,
-'VkPhysicalDeviceGroupPropertiesKHX' : 431,
-'VkExternalImageFormatPropertiesKHR' : 432,
-'VkPhysicalDeviceIDPropertiesKHR' : 433,
+'VkPhysicalDeviceMultiviewProperties' : 430,
+'VkPhysicalDeviceGroupProperties' : 431,
+'VkExternalImageFormatProperties' : 432,
+'VkPhysicalDeviceIDProperties' : 433,
 'VkPhysicalDeviceMultiviewPerViewAttributesPropertiesNVX' : 434,
 'VkHdrMetadataEXT' : 435,
-'VkExternalMemoryPropertiesKHR' : 436,
+'VkExternalMemoryProperties' : 436,
 'VkFormatProperties' : 437,
 'VkImageFormatProperties' : 438,
 'VkPhysicalDeviceLimits' : 439,
@@ -497,10 +497,10 @@ func_struct_id_map = {
 'VkMemoryType' : 441,
 'VkMemoryHeap' : 442,
 'VkSparseImageFormatProperties' : 443,
-'VkSurfaceCapabilitiesKHR' : 444,
-'VkDisplayPropertiesKHR' : 445,
-'VkDisplayPlaneCapabilitiesKHR' : 446,
-'VkSharedPresentSurfaceCapabilitiesKHR' : 447,
+'VkSurfaceCapabilities' : 444,
+'VkDisplayProperties' : 445,
+'VkDisplayPlaneCapabilities' : 446,
+'VkSharedPresentSurfaceCapabilities' : 447,
 'VkExternalImageFormatPropertiesNV' : 448,
 'VkPhysicalDeviceBlendOperationAdvancedFeaturesEXT' : 449,
 'VkPhysicalDeviceBlendOperationAdvancedPropertiesEXT' : 450,
@@ -510,36 +510,36 @@ func_struct_id_map = {
 'VkPipelineCoverageToColorStateCreateInfoNV' : 454,
 'VkSamplerReductionModeCreateInfoEXT' : 455,
 'VkPhysicalDeviceProperties' : 456,
-'VkSurfaceFormatKHR' : 457,
-'VkExportFenceCreateInfoKHR' : 458,
-'VkPhysicalDeviceExternalFenceInfoKHR' : 459,
-'VkExternalFencePropertiesKHR' : 460,
-'vkGetPhysicalDeviceExternalFencePropertiesKHR' : 461,
-'VkImportFenceFdInfoKHR' : 462,
-'VkFenceGetFdInfoKHR' : 463,
-'vkImportFenceFdKHR' : 464,
-'vkGetFenceFdKHR' : 465,
-'VkImportFenceWin32HandleInfoKHR' : 466,
-'VkExportFenceWin32HandleInfoKHR' : 467,
-'VkFenceGetWin32HandleInfoKHR' : 468,
-'vkImportFenceWin32HandleKHR' : 469,
-'vkGetFenceWin32HandleKHR' : 470,
-'VkSemaphoreGetFdInfoKHR' : 471,
-'VkSemaphoreGetWin32HandleInfoKHR' : 472,
-'VkMemoryGetFdInfoKHR' : 473,
-'VkMemoryGetWin32HandleInfoKHR' : 474,
-'VkMemoryDedicatedRequirementsKHR' : 475,
-'VkMemoryDedicatedAllocateInfoKHR' : 476,
-'VkBufferMemoryRequirementsInfo2KHR' : 477,
-'VkImageMemoryRequirementsInfo2KHR' : 478,
-'VkImageSparseMemoryRequirementsInfo2KHR' : 479,
-'VkMemoryRequirements2KHR' : 480,
-'VkSparseImageMemoryRequirements2KHR' : 481,
-'vkGetImageMemoryRequirements2KHR' : 482,
-'vkGetBufferMemoryRequirements2KHR' : 483,
-'vkGetImageSparseMemoryRequirements2KHR' : 484,
-'VkPhysicalDevice16BitStorageFeaturesKHR' : 485,
-'VkPhysicalDeviceVariablePointerFeaturesKHR' : 486,
+'VkSurfaceFormat' : 457,
+'VkExportFenceCreateInfo' : 458,
+'VkPhysicalDeviceExternalFenceInfo' : 459,
+'VkExternalFenceProperties' : 460,
+'vkGetPhysicalDeviceExternalFenceProperties' : 461,
+'VkImportFenceFdInfo' : 462,
+'VkFenceGetFdInfo' : 463,
+'vkImportFenceFd' : 464,
+'vkGetFenceFd' : 465,
+'VkImportFenceWin32HandleInfo' : 466,
+'VkExportFenceWin32HandleInfo' : 467,
+'VkFenceGetWin32HandleInfo' : 468,
+'vkImportFenceWin32Handle' : 469,
+'vkGetFenceWin32Handle' : 470,
+'VkSemaphoreGetFdInfo' : 471,
+'VkSemaphoreGetWin32HandleInfo' : 472,
+'VkMemoryGetFdInfo' : 473,
+'VkMemoryGetWin32HandleInfo' : 474,
+'VkMemoryDedicatedRequirements' : 475,
+'VkMemoryDedicatedAllocateInfo' : 476,
+'VkBufferMemoryRequirementsInfo2' : 477,
+'VkImageMemoryRequirementsInfo2' : 478,
+'VkImageSparseMemoryRequirementsInfo2' : 479,
+'VkMemoryRequirements2' : 480,
+'VkSparseImageMemoryRequirements2' : 481,
+'vkGetImageMemoryRequirements2' : 482,
+'vkGetBufferMemoryRequirements2' : 483,
+'vkGetImageSparseMemoryRequirements2' : 484,
+'VkPhysicalDevice16BitStorageFeatures' : 485,
+'VkPhysicalDeviceVariablePointerFeatures' : 486,
 'VkSampleLocationsInfoEXT' : 487,
 'VkRenderPassSampleLocationsBeginInfoEXT' : 488,
 'VkPipelineSampleLocationsStateCreateInfoEXT' : 489,
@@ -555,22 +555,52 @@ func_struct_id_map = {
 'vkMergeValidationCachesEXT' : 499,
 'VkAttachmentSampleLocationsEXT' : 500,
 'VkSubpassSampleLocationsEXT' : 501,
-'VkPhysicalDevicePointClippingPropertiesKHR' : 502,
-'VkInputAttachmentAspectReferenceKHR' : 503,
-'VkRenderPassInputAttachmentAspectCreateInfoKHR' : 504,
-'VkImageViewUsageCreateInfoKHR' : 505,
-'VkPipelineTessellationDomainOriginStateCreateInfoKHR' : 506,
-'VkImageFormatListCreateInfoKHR' : 507,
-'VkSamplerYcbcrConversionCreateInfoKHR' : 508,
-'VkBindImagePlaneMemoryInfoKHR' : 509,
-'VkImagePlaneMemoryRequirementsInfoKHR' : 510,
-'vkCreateSamplerYcbcrConversionKHR' : 511,
-'VkBindBufferMemoryDeviceGroupInfoKHX' : 512,
-'VkBindImageMemoryDeviceGroupInfoKHX' : 513,
-'vkDestroySamplerYcbcrConversionKHR' : 514,
-'VkPhysicalDeviceSamplerYcbcrConversionFeaturesKHR' : 515,
-'VkSamplerYcbcrConversionImageFormatPropertiesKHR' : 516,
-'VkSamplerYcbcrConversionInfoKHR' : 517,
+'VkPhysicalDevicePointClippingProperties' : 502,
+'VkInputAttachmentAspectReference' : 503,
+'VkRenderPassInputAttachmentAspectCreateInfo' : 504,
+'VkImageViewUsageCreateInfo' : 505,
+'VkPipelineTessellationDomainOriginStateCreateInfo' : 506,
+'VkImageFormatListCreateInfo' : 507,
+'VkSamplerYcbcrConversionCreateInfo' : 508,
+'VkBindImagePlaneMemoryInfo' : 509,
+'VkImagePlaneMemoryRequirementsInfo' : 510,
+'vkCreateSamplerYcbcrConversion' : 511,
+'VkBindBufferMemoryDeviceGroupInfo' : 512,
+'VkBindImageMemoryDeviceGroupInfo' : 513,
+'vkDestroySamplerYcbcrConversion' : 514,
+'VkPhysicalDeviceSamplerYcbcrConversionFeatures' : 515,
+'VkSamplerYcbcrConversionImageFormatProperties' : 516,
+'VkSamplerYcbcrConversionInfo' : 517,
+'VkDeviceQueueGlobalPriorityCreateInfoEXT' : 518,
+'vkGetShaderInfoAMD' : 519,
+'VkShaderStatisticsInfoAMD' : 520,
+'VkImportMemoryHostPointerInfoEXT' : 521,
+'VkMemoryHostPointerPropertiesEXT' : 522,
+'VkPhysicalDeviceExternalMemoryHostPropertiesEXT' : 523,
+'vkGetMemoryHostPointerPropertiesEXT' : 524,
+'VkPhysicalDeviceConservativeRasterizationPropertiesEXT' : 525,
+'VkPipelineRasterizationConservativeStateCreateInfoEXT' : 526,
+'vkCmdWriteBufferMarkerAMD' : 527,
+'VkDescriptorSetLayoutSupport' : 528,
+'VkDeviceQueueInfo2' : 529,
+'VkPhysicalDeviceMaintenance3Properties' : 530,
+'VkPhysicalDeviceProtectedMemoryFeatures' : 531,
+'VkPhysicalDeviceProtectedMemoryProperties' : 532,
+'VkPhysicalDeviceShaderDrawParameterFeatures' : 533,
+'VkPhysicalDeviceSubgroupProperties' : 534,
+'VkProtectedSubmitInfo' : 535,
+'vkEnumerateInstanceVersion' : 536,
+'vkGetDescriptorSetLayoutSupport' : 537,
+'vkGetDeviceQueue2' : 538,
+'VkDebugUtilsObjectNameInfoEXT' : 539,
+'VkDebugUtilsObjectTagInfoEXT' : 540,
+'VkDebugUtilsLabelEXT' : 541,
+'VkDebugUtilsMessengerCallbackDataEXT' : 542,
+'VkDebugUtilsMessengerCreateInfoEXT' : 543,
+'vkCreateDebugUtilsMessengerEXT' : 544,
+'vkSubmitDebugUtilsMessageEXT' : 545,
+'VkPhysicalDeviceVertexAttributeDivisorPropertiesEXT' : 546,
+'VkPipelineVertexInputDivisorStateCreateInfoEXT' : 547,
 ### ADD New func/struct mappings above this line
 }
 # Mapping of params to unique IDs
@@ -1075,6 +1105,30 @@ implicit_param_map = {
 'ycbcrConversion' : 497,
 'pViewFormats' : 498,
 'conversion' : 499,
+'pPostSubpassSampleLocations' : 500,
+'globalPriority' : 501,
+'shaderStage' : 502,
+'infoType' : 503,
+'pInfoSize' : 504,
+'shaderStageMask' : 505,
+'pMemoryHostPointerProperties' : 506,
+'pHostPointer' : 507,
+'conservativeRasterizationMode' : 508,
+'pViewports' : 509,
+'pViewportWScalings' : 510,
+'pSplitInstanceBindRegions' : 511,
+'pApiVersion' : 512,
+'pSupport' : 513,
+'pQueueInfo' : 514,
+'splitInstanceBindRegionCount' : 515,
+'pLabelName' : 516,
+'messageSeverity' : 517,
+'messageType' : 518,
+'pfnUserCallback' : 519,
+'pMessenger' : 520,
+'messageTypes' : 521,
+'vertexBindingDivisorCount' : 522,
+'pVertexBindingDivisors' : 523,
 ### ADD New implicit param mappings above this line
 }
 
@@ -1083,19 +1137,22 @@ uniqueid_set = set() # store uniqueid to make sure we don't have duplicates
 # Convert a string VUID into numerical value
 #  See "VUID Mapping Details" comment above for more info
 def convertVUID(vuid_string):
-    """Convert a string-based VUID into a numberical value"""
+    """Convert a string-based VUID into a numerical value"""
     #func_struct_update = False
     #imp_param_update = False
     if vuid_string in ['', None]:
         return -1
     vuid_parts = vuid_string.split('-')
+    # Alias core/KHR/KHX ids because not all VUIDs in the spec get updated at the same time
+    if vuid_parts[1].endswith('KHR') or vuid_parts[1].endswith('KHX'):
+        vuid_parts[1] = vuid_parts[1][:-3]
     if vuid_parts[1] not in func_struct_id_map:
         print ("ERROR: Missing func/struct map value for '%s'!" % (vuid_parts[1]))
         print (" TODO: Need to add mapping for this to end of func_struct_id_map")
         print ("   replace '### ADD New func/struct mappings above this line' line with \"'%s' : %d,\"" % (vuid_parts[1], len(func_struct_id_map)))
         func_struct_id_map[vuid_parts[1]] = len(func_struct_id_map)
         #func_struct_update = True
-        sys.exit()
+        sys.exit(1)
     uniqueid = func_struct_id_map[vuid_parts[1]] << FUNC_STRUCT_SHIFT
     if vuid_parts[-1].isdigit(): # explit VUID has int on the end
         explicit_id = int(vuid_parts[-1])
@@ -1104,7 +1161,7 @@ def convertVUID(vuid_string):
     else: # implicit case
         if vuid_parts[-1] not in implicit_type_map:
             print("ERROR: Missing mapping for implicit type '%s'!\nTODO: Please add new mapping." % (vuid_parts[-1]))
-            sys.exit()
+            sys.exit(1)
         else:
             param_id = 0 # Default when no param is available
             if vuid_parts[-2] != vuid_parts[1]: # we have a parameter
@@ -1115,7 +1172,7 @@ def convertVUID(vuid_string):
                     print ("   replace '### ADD New implicit param mappings above this line' line with \"'%s' : %d,\"" % (vuid_parts[-2], len(implicit_param_map)))
                     implicit_param_map[vuid_parts[-2]] = len(implicit_param_map)
                     #imp_param_update = True
-                    sys.exit()
+                    sys.exit(1)
                 uniqueid = uniqueid + (param_id << IMPLICIT_PARAM_SHIFT) + (implicit_type_map[vuid_parts[-1]] << IMPLICIT_TYPE_SHIFT) + implicit_bit0
             else: # No parameter so that field is 0
                 uniqueid = uniqueid + (implicit_type_map[vuid_parts[-1]] << IMPLICIT_TYPE_SHIFT) + implicit_bit0
